use actix_web::{main, web::Data, App, HttpServer};
use actix_web::middleware::NormalizePath;

mod database;
mod config;
mod schema;
mod utils;
mod v1;

#[derive(Clone)]
pub(crate) struct State {
    db: database::DbPool,
    config: config::Config
}

#[main]
async fn main() -> std::io::Result<()> {
    let config = config::read_config();
    let conn = database::establish_connection(config.clone().database_url);
    
    let state = State {
        db: conn,
        config: config.clone()
    };

    let server = HttpServer::new(move || {
        let state = state.clone();

        App::new()
            .wrap(NormalizePath::trim())
            .app_data(Data::new(state))
            .configure(v1::init_routes)
    })
    .bind((config.host, config.port))?
    .run();

    println!("API running at on port {}", config.port);

    server.await
}
