use actix_web::{
    post,
    web::{scope, Data, Json, ServiceConfig},
    HttpResponse, Responder,
};
use argon2::verify_encoded;
use diesel::{ExpressionMethods, OptionalExtension, QueryDsl, RunQueryDsl};

use crate::utils::response::{failure, success};
use crate::State;
use crate::{database::models::user::NewUser, schema::users::dsl::*};
use crate::{database::models::user::User, utils::jwt::Claims};

#[post("/login")]
async fn login(state: Data<State>, login_data: Json<NewUser>) -> impl Responder {
    let mut conn = match state.db.get() {
        Ok(conn) => conn,
        Err(why) => {
            return HttpResponse::InternalServerError()
                .body(format!("database connection error: {}", why))
        }
    };

    let secret = &state.config.jwt_secret;
    let user_result = users
        .filter(username.eq(&login_data.username))
        .first::<User>(&mut conn)
        .optional();

    match user_result {
        Ok(Some(user)) => match verify_encoded(&user.password, login_data.password.as_bytes()) {
            Ok(true) => {
                let token = Claims::generate_token(secret.as_str(), user.id);

                HttpResponse::Ok().json(success(token))
            }
            Ok(false) => HttpResponse::Unauthorized().json(failure("invalid username or password")),
            Err(err) => HttpResponse::InternalServerError()
                .json(failure(format!("error while verifying password: {}", err))),
        },
        Ok(None) | Err(_) => {
            HttpResponse::Unauthorized().json(failure("invalid username or password"))
        }
    }
}

pub fn init_routes(cfg: &mut ServiceConfig) {
    cfg.service(scope("/auth").service(login));
}
